require 'tmpdir'
class ConfigObject
	def initialize(config_file)
		@config_file = config_file
	end

	def configure
		File.open(@config_file, 'a') do |cfg_file|
			cfg_file.write self.to_s
		end
	end

	def to_s
		raise "Not supported!"
	end
end

class HostGroup < ConfigObject
	def initialize(config_path, name, hosts)
		super(File.join config_path, "etc", "hostgroups.cfg")
		@config_path = config_path
		@name = name
		@hosts = hosts
	end

	def configure
		super
		@hosts.each {|host| Host.new(@config_path, host, @name).configure}
	end

	def to_s
<<eos

define hostgroup{
	hostgroup_name	#{@name}
	alias			#{@name}
}
eos
	end
end
class Host < ConfigObject
	def initialize(config_path, name, group)
		super(File.join config_path, "etc", "hosts.cfg")
		@name = name
		@group = group
	end

	def to_s
<<eos

define host {
	use			default-host-template
	host_name	#{@name}
	alias		#{@name}
	hostgroups	#{@group}
	check_command	check-host-alive
}
eos
	end
end

class Service < ConfigObject
	def initialize(config_path, hash)
		super(File.join config_path, "etc", "services.cfg")
		@description = hash['Description']
		@host = hash['Host']
		@command = hash['Check command']
		@notifications = hash['Notifications'] == 'Enabled' ? 1 : 0
		@active_checks = hash['Active checks'] == 'Enabled' ? 1 : 0
	end

	def to_s
<<eos

define service{
	use					default-service
	host_name			#{@host}
	service_description #{@description}
	check_command		#{@command}
	notifications_enabled	#{@notifications}
	active_checks_enabled 	#{@active_checks}
}
eos
	end
end

class NagiosConfiguration
	def initialize()
		@source_path = "test/configs/templates/."
		@host_check_results = []
		@cmd_file = "/tmp/ninja-test/nagios.cmd"
	end

	#Writers
	#NOTE: There is no setter/writer for the hosts
	#instance variable. Users are instead expected to provide whatever
	#hosts they wish to be included in the configuration as part of
	#one or more hostgroups.
	def hostgroups=(hostgroups)
		@hostgroups = hostgroups
	end

	def services=(services)
		@services = services
	end

	#Pseudo readers that return an array of strings to look for
	#on a web page
	def services
		@services.map{ |service|
			service['Description']
		}
	end

	def hosts
		@hosts
	end

	def hostgroups
		@hostgroups.map{ |group|
			group['Name']
		}
	end

	def add_host_check_result(result)
		@host_check_results.push "PROCESS_HOST_CHECK_RESULT;%s" % result
	end

	def add_service_check_result(result)
		@host_check_results.push "PROCESS_SERVICE_CHECK_RESULT;%s" % result
	end

	def submit_passive_check_results
		@host_check_results.each {|result|
			issue_command result
		}
		@host_check_results = []
	end

	def activate
		@config_path = Dir.mktmpdir("config_")
		Dir.mkdir("/tmp/ninja-test", 0777)
		Dir.mkdir(File.join(@config_path, "var"), 0777)
		FileUtils.chmod 0777, @config_path
		FileUtils.chmod 0777, File.join(@config_path, "var")
		FileUtils.chmod 0777, "/tmp/ninja-test"
		sock_path = 'unix:///tmp/ninja-test/live'
		FileUtils.cp_r @source_path, @config_path
		nagios_cfg = File.join @config_path, "etc", "nagios.cfg"
		merlin_cfg = File.join @config_path, "etc", "merlin.conf"
		update_paths nagios_cfg
		update_paths merlin_cfg
		configure_all
		set_nagios_pipe @cmd_file
		spawn_child "/opt/monitor/op5/merlin/merlind -c %s" % merlin_cfg
		spawn_child "/opt/monitor/bin/monitor -d %s" % nagios_cfg
		file_wait(@cmd_file)
		file_wait("/tmp/ninja-test/live")
		check_livestatus sock_path
	end

	def check_livestatus(ls_sock_path)
		IO.popen("python2.6 test/cucumber/helpers/support/check_livestatus.py " + ls_sock_path) { |p|
			livestatus_check_output = p.read

		}
		if ($?.exitstatus != 0)
			puts livestatus_check_output
			raise "Livestatus not up :("
		end
	end

	def verify_configuration(cfg)
		nagios_cfg = File.join(@config_path, "etc", "nagios.cfg")
		IO.popen("/opt/monitor/bin/monitor -v " + cfg) {|p|
			config_check_output = p.read
		}
		if ($?.exitstatus != 0)
			puts config_check_output
			raise "Faulty configuration :("
		end
	end

	def spawn_child(cmdstr)
		pid = fork
		pid.nil? ? exec(cmdstr) : Process.detach(pid)
	end

	def update_paths(cfg_path)
		contents = File.read(cfg_path)
		File.open(cfg_path, 'w') do |cfg_file|
			cfg_file.write contents.gsub /@@TESTDIR@@/, @config_path
		end
	end

	def set_nagios_pipe(pipe_path)
		config_php = "/opt/monitor/op5/ninja/application/config/custom/config.php"
		if File.exist? config_php
			raise "Cowardly refusing to overwrite existing custom config at %s" % config_php
		else
			File.open(config_php, 'w') { |config_file|
				config_file.write "<?php $config['nagios_pipe'] = '%s'; putenv('OP5LIBCFG=%s/test/configs/all-host_service-states/op5lib');" % [pipe_path, Dir.pwd];
			}
		end
	end

	def configure_all
		@hosts = Set.new
		@hostgroups.each do |group|
			grouphosts = group['Hosts'].split(",")
			HostGroup.new(@config_path, group['Name'], grouphosts).configure()
			@hosts = @hosts.merge(grouphosts)
		end
		@services.each do |service_hash|
			Service.new(@config_path, service_hash).configure()
		end
	end

	def issue_command(command)
		if File.exists? @cmd_file
			File.open(@cmd_file, 'w+') do |cmd_f|
				cmd_f.puts("[%d] %s" % [Time.now.to_i, command])
				cmd_f.flush
			end
		else
			raise "Command file not available, did Nagios start properly?"
		end
	end

	def reset(cleanup_dirs=true)
		File.unlink "/opt/monitor/op5/ninja/application/config/custom/config.php"
		begin
			merlin_pid = File.read(File.join @config_path, "var", "merlin.pid").to_i
			Process.kill("TERM", merlin_pid)
		rescue Errno::ESRCH
			puts "Can not kill Merlin, already dead"
		end
		nagios_lock = File.join @config_path, "var", "nagios.lock"
		begin
			nagios_pid = File.read(nagios_lock).to_i
			Process.kill("TERM", nagios_pid)
		rescue Errno::ESRCH
			puts "Can not kill Nagios, already dead"
		end
		file_wait(nagios_lock, :state=>:removed)
		#We need to wait for the livestatus socket to disappear
		#since Nagios has not necessarily finished unloading all
		#brokers by the time it removes the pid file (above)
		file_wait("/tmp/ninja-test/live", {:state=>:removed})
		FileUtils.rm_rf("/tmp/ninja-test")
		if cleanup_dirs
			FileUtils.rm_rf @config_path if File.exists? @config_path
		else
			puts "Not removing directory %s" % @config_path
		end
	end

	def file_wait(file, args = {:state => :exists, :verbose => false})
		if args[:state].equal? :exists
			cond = lambda {|file| File.exists?(file)}
		elsif args[:state].equal? :removed
			cond = lambda {|file| not File.exists?(file)}
		end
		max_sleep = 60
		sleep_time = 0.1
		slept = 0
		until cond.call(file) do
			if args[:verbose]
				puts "Waiting for file %s" % [file.to_s]
			end
			if slept < max_sleep
				sleep sleep_time
				slept += sleep_time
			else
				raise "Timed out waiting for file %s" % [file.to_s]
			end
		end
	end

end
