<?php

require_once('devtools/tapunit.php');
require_once('classes/object_class.php');

/**
 * Test higher level functionality in object_class
 * Lower level tests are in oatest.php
 */
class ObjectTest extends TapUnit
{
	function testTemplateDeletion()
	{
		$tmpl = new host_template_class();
		$tmpl->get_default_object();
		$tmpl->obj['name'] = 'tmp_tmpl';
		$tmpl->obj['check_command'] = $tmpl->obj['notification_period'] = 1;
		$tmpl->save_object();
		$this->ok_empty($tmpl->validation_error_list, 'Template created');
		$tmpl_id = $tmpl->id;

		$host = new host_class();
		$host->get_default_object();
		$host->obj['host_name'] = $host->obj['alias'] = $host->obj['address'] = 'tmphost';
		$host->obj['template'] = $tmpl->id;
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'Host created');
		$host_id = $host->id;

		$this->ok(!$tmpl->banish_object(), "Template isn't deleted because it has an active object user");
		$this->ok($host->banish_object(), "Host is deleted");
		$this->ok($tmpl->banish_object(), "Template is deleted when host is gone");
		
		$tmpl = new host_template_class($tmpl_id);
		$this->ok_empty($tmpl->obj, 'Template deleted');

		$host = new host_class($host_id);
		unset($host->obj['contacts']);
		$this->ok_empty($host->obj, 'Host deleted when it\'s template is');
	}

	function testCopyService()
	{
		$srv = new service_class();
		$srv->get_default_object();
		$srv->obj['host_name'] = 1;
		$srv->obj['service_description'] = 'testservice';
		$srv->obj['check_command'] = 1;
		$srv->save_object();
		$this->ok_empty($srv->validation_error_list, 'Service saved ok');

		$esc = new serviceescalation_class();
		$esc->get_default_object();
		$esc->obj['service'] = $srv->id;
		$esc->save_object();
		$this->ok_empty($esc->validation_error_list, 'Escalation saved ok');

		$new_srv = $srv->clone_object();
		$new_srv->obj['service_description'] = 'cloned testservice';
		$this->ok_eq($new_srv->clone_id, $srv->id, 'New service has old service\'s id as clone_id');
		$tmp = $srv->find_slave_objects();
		$this->ok_eq($tmp['serviceescalation'][0], $esc->id, 'Old service has the new escalation');
		$this->ok_empty($new_srv->find_slave_objects(), 'No dependant objects yet');
		$new_srv->save_object();
		$this->ok_empty($new_srv->validation_error_list, 'New service saved ok');
		$tmp = $srv->find_slave_objects();
		$this->ok_eq($tmp['serviceescalation'][0], $esc->id, 'Old service still has the new escalation');
		$tmp2 = $new_srv->find_slave_objects();
		$this->ok(!empty($tmp2['serviceescalation']), 'New service has its own service escalation');
		$this->ok($tmp2['serviceescalation'][0] != $tmp['serviceescalation'][0], 'Different escalations are assigned to the two services');
	}

	function testExtinfo() {
		$hextinfo = new hostextinfo_class(41);
		$this->ok(!empty($hextinfo->obj), 'Hostextinfo exists');
		$host = new host_class($hextinfo->obj['host_name']);
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'Host saved ok');
		$new_hostextinfo = new hostextinfo_class($hextinfo->id);
		$this->ok_empty($new_hostextinfo->obj, 'Hostextinfo no longer exists');
		$this->ok_eq($hextinfo->obj['notes_url'], $host->obj['notes_url'], "Notes URL moved to master object");
		$this->ok_eq($hextinfo->obj['icon_image'], $host->obj['icon_image'], "Icon image moved to master object");
		$this->ok_eq($hextinfo->obj['icon_image_alt'], $host->obj['icon_image_alt'], "Icon image alt moved to master object");
		$this->ok_eq($hextinfo->obj['statusmap_image'], $host->obj['statusmap_image'], "Statusmap image moved to master object");
	}

	function group_of_groups_helper($type, $groupmember, $member) {
		$class = $type.'_class';
		$obj = new $class();
		$obj->get_default_object();
		$obj->obj[$type.'_name'] = 'test '.$type;
		$obj->obj['alias'] = 'test '.$type;
		$this->ok($obj->save_object(), "Tried to save a {$type} with no members, but failed");
		$obj->obj['members'] = array($member);
		$this->ok($obj->save_object(), "Saving is successful when members are added to the {$type}");
		$reloaded_obj = new $class($obj->id);
		$this->ok_eq($reloaded_obj->obj['members'], $obj->obj['members'], 'Members are saved correctly');
		$this->ok(!isset($reloaded_obj->obj[$type.'_members']), "No {$type}_members added");
		$obj->obj['members'] = array();
		$obj->obj[$type.'_members'] = array($groupmember);
		$this->ok($obj->save_object(), "Saving is successful when {$type}_members");
		$reloaded_obj = new $class($obj->id);
		$this->ok_eq($reloaded_obj->obj[$type.'_members'], $obj->obj[$type.'_members'], "{$type}_members are saved correctly");
		$this->ok(!isset($reloaded_obj->obj['members']), "No members added");

		$clone = $obj->clone_object();
		$this->ok_eq($clone->obj[$type.'_members'], $obj->obj[$type.'_members'], "Cloned object retains {$type}_members attribute");
		$id = $obj->id;
		$obj->banish_object();
		$new_obj = new $class($id);
		$this->ok_empty($new_obj->obj, 'Object was really deleted');
	}

	function testGroupOfGroups() {
		$this->group_of_groups_helper('hostgroup', 1, 1);
		$this->group_of_groups_helper('servicegroup', 1, 1);
		$this->group_of_groups_helper('contactgroup', 1, 1);
	}

	function template_helper($type) {
		$props = $type == 'contact' ? array('address1', 'address2') : array('notes', 'action_url');

		$obj_class = $type.'_class';
		$tmpl_class = $type.'_template_class';
		$gptmpl = new $tmpl_class();
		$gptmpl->obj['name'] = 'Grandparent test template';
		$gptmpl->obj[$props[0]] = "This is a {$props[0]} from grandparent";
		$this->ok($gptmpl->save_object(), "Saving grandparent $type template should work, got " . var_export($gptmpl->validation_error_list, true));
		$ptmpl = new $tmpl_class();
		$ptmpl->obj['name'] = 'Parent test template';
		$ptmpl->obj[$props[1]] = 'http://schwarzenegger.com';
		$ptmpl->obj['template'] = $gptmpl->id;
		$this->ok($ptmpl->save_object(), "Saving parent $type template should work, got " . var_export($ptmpl->validation_error_list, true));
		$obj = new $obj_class();
		$obj->obj['template'] = $ptmpl->id;
		if ($type == 'service') {
			$obj->obj['host_name'] = 'host_9';
			$obj->obj['service_description'] = 'test service';
			$obj->obj['check_interval'] = $obj->obj['retry_interval'] = $obj->obj['check_period'] = 1;
		} else {
			$obj->obj[$type.'_name'] = 'test '.$type;
		}
		if ($type !== 'contact')
			$obj->obj['max_check_attempts'] = $obj->obj['notification_interval'] = $obj->obj['notification_period'] = 1;
		$this->ok($obj->save_object(), "Saving the $type object should work, got " . var_export($obj->validation_error_list, true));
		$obj = new $obj_class($obj->id);
		$obj->expand_object(true);
		$ptmpl->expand_object(true);
		$gptmpl->expand_object(true);
		$this->ok_eq($obj->obj[$props[1]], $ptmpl->obj[$props[1]], $type . ' action url should be set from direct template');
		$this->ok_eq($obj->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set from indirect template');
		$this->ok_eq($ptmpl->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set to template from parent template');

		$this->ok(!$gptmpl->delete_object(), 'Deleting grandparent should not work, but it did: ' .$gptmpl->delete_message);
		$this->ok(!$ptmpl->delete_object(), 'Deleting parent should not work, but it did: ' .$ptmpl->delete_message);
		$this->ok($obj->delete_object(), "Deleting object should now work, but it didn't: " .$obj->delete_message);
		$this->ok($ptmpl->delete_object(), "Deleting parent should now work, but it didn't: " .$ptmpl->delete_message);
		$this->ok($gptmpl->delete_object(), "Deleting grandparent should now work, but it didn't: " .$gptmpl->delete_message);
	}

	function testTemplateTemplate() {
		$this->template_helper('host');
		$this->template_helper('service');
		$this->template_helper('contact');
	}

	function testResaveServiceWithSemicolons() {
		$svc = new service_class('host_9;HTTP Server');
		$svc->save_object();
		$this->ok_empty($svc->validation_error_list, 'Resave service with escaped ; ok');
		$svc->obj['check_command'] = 'check_http!arg;with;unescaped;semicolons';
		$this->ok(!$svc->save_object(), 'Save service with unescaped ; not ok');
		$svc->obj['check_command'] = 'check_http!arg\;with;semicolons';
		$this->ok(!$svc->save_object(), 'Save service with some unescaped ; not ok');
	}

	function testSaveAndPermissions() {
		$host = new host_class('host_9');
		$svc = new service_class();
		$svc->obj['host_name'] = $host->id;
		$svc->obj['service_description'] = 'testservice';
		$svc->obj['template'] = 1;
		$svc->obj['check_command'] = 1;
		$svc->save_object();
		$svc->prepare_draw();
		$this->ok_empty($svc->validation_error_list, 'New service saved');
		$fellow_slaves = $svc->get_fellow_slaves();
		$this->ok(count($fellow_slaves) > 1, 'The list of fellow slaves contains objects other than this');
		$old_svc = new service_class('host_9;HTTP Server');
		$this->ok(in_array($old_svc->id, array_keys($fellow_slaves)), 'The HTTP Server service on this host is a fellow slave');
		$svc->banish_object();
	}

	function testDoubleNormalize() {
		$host = new host_class('wooopidoo');
		$host->get_default_object();
		$host->obj['contact_groups'] = array('admins');
		$host->normalize();
		$host->normalize();
		$this->ok(true, 'Double-normalize does not break mselect values');
	}

	/**
	 * bug #6488
	 */
	function testPropagateAppend() {
		$host = new host_class('host_9');
		$host->propagate_variables(array('hostgroups' => array('hostgroup_3')), array('host_11', 'host_23'), 'append');
		$propagated = new host_class('host_23');
		$this->ok_eq(count($propagated->obj['hostgroups']), 2, 'host_23 should have its original + host_9\'s hostgroup = 2');
		$this->ok(!in_array('hostgroup_4', $propagated->obj['hostgroups']), 'host_23 should not get host_11s old hostgroups');
	}

	function testUnicode() {
		$strings = array(
			'今年から貝が胃に棲み始めました。' => true, #kanji - with scary full stop
			'আমার হভারক্রাফ্ট ইল-এ ভর্তি হয়ে' => true, #bengal - scary all over, with "vowel stop" and nastiness
			'  ' => true, #klingon - private plane
			"Looks alright (but isn't)" => false, #parenthesis
			"Now we backspace\x08" => false, #backspace character
		);

		$host = new host_class('new host');
		$host->get_default_object();
		foreach ($strings as $string => $legal) {
			$host->obj['host_name'] = $string;
			$this->ok($legal == $host->save_object(), "Setting the identifier of a host to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($host->validation_error_list, true));
		}

		$service = new service_class();
		$service->obj['template'] = 1;
		$service->obj['check_command'] = 1;
		$service->obj['host_name'] = $host->id;
		foreach ($strings as $string => $legal) {
			$service->obj['service_description'] = $string;
			$this->ok($legal == $service->save_object(), "Setting the identifier of a service to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($service->validation_error_list, true));
		}
	}

	function testContactWithUser() {
		$contact = new contact_class();
		$contact->obj['enable_access'] = 1;
		$contact->obj['contact_name'] = 'test_contact_with_user';
		$contact->obj['password'] = 'test_password';
		$contact->obj['groups'] = array('contact_limited');
		$this->ok($contact->save_object(), "Saving contact with test user should work, got " . var_export($contact->validation_error_list, true));
		$user = new user_class('test_contact_with_user');
		$this->ok_eq($user->obj['groups'], $contact->obj['groups'], "Group list for loaded user should be equal to group list for contact-with-user");
	}

	# These pnp xml files exist: host17 (host itself, Bandwidth, Mem Usage), host14 (host itself, PING)

	function testGraphCollections() {
		$obj = new graph_collection_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid collection should return empty object');
		$this->ok(!$obj->id, 'Loading invalid collection should return false id');

		$obj = new graph_collection_class();
		$obj->obj['page_name'] = 'test1';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$obj->obj['use_regex'] = false;
		$this->ok($obj->save_object(), 'Saving collection should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same collection twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$obj->obj['use_regex'] = false;
		$this->ok($obj->save_object(), 'Resaving collection with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$obj->obj['page_name'] = 'test2';
		$this->ok($obj->save_object(), 'Saving collection with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test2');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$this->ok_empty($obj->obj, 'Loading old name for renamed collection should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed collection should return false id');

		$obj = new graph_collection_class('test2');
		$this->ok($obj->delete_object(), 'Object deletion should work');
		$obj = new graph_collection_class('test2');
		$this->ok_empty($obj->obj, 'Loading deleted collection should return empty object');
		$this->ok(!$obj->id, 'Loading deleted collection should return false id');
	}

	function testCombinedGraph() {
		$obj = new combined_graph_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading invalid combined graph should return false id');

		$obj = new combined_graph_class();
		$obj->obj['title'] = 'test1';
		$obj->obj['comment'] = 'yes';
		$obj->obj['type'] = 'line2';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$this->ok($obj->save_object(), 'Saving combined graph should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same combined graph twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$this->ok($obj->save_object(), 'Resaving combined graph with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$obj->obj['title'] = 'test2';
		$this->ok($obj->save_object(), 'Saving combined graph with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test2');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$this->ok_empty($obj->obj, 'Loading old name for renamed combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed combined graph should return false id');

		$obj = new combined_graph_class('test2');
		$this->ok($obj->delete_object(), 'Object deletion should work');
		$obj = new combined_graph_class('test2');
		$this->ok_empty($obj->obj, 'Loading deleted combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading deleted combined graph should return false id');
	}

	function testGraphTemplate() {
		$obj = new graph_template_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid graph template should return empty object');
		$this->ok(!$obj->id, 'Loading invalid graph template should return false id');

		$obj = new graph_template_class();
		$obj->obj['check'] = 'check-host-alive';
		$obj->obj['graph_template'] = 'templates.dist/check_nt_mem.php';
		$this->ok($obj->save_object(), 'Saving graph template should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same graph template twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check-host-alive');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$obj->obj['graph_template'] = 'templates.dist/check_nwstat_LOAD1.php';
		$this->ok($obj->save_object(), 'Resaving graph template with different template should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check-host-alive');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$obj->obj['check'] = 'check_ssh';
		$this->ok($obj->save_object(), 'Saving graph template with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check_ssh');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$this->ok_empty($obj->obj, 'Loading old name for renamed graph template should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed graph template should return false id');

		$obj = new graph_template_class('check_ssh');
		$this->ok($obj->delete_object(), 'Object deletion should work, but got: '.$obj->delete_message);
		$obj = new graph_template_class('check_ssh');
		$this->ok_empty($obj->obj, 'Loading deleted graph template should return empty object');
		$this->ok(!$obj->id, 'Loading deleted graph template should return false id');
	}
}

if (empty($main_tap))
	$main_tap = new phptap('Object tests');

runUnit('ObjectTest', $main_tap);
